#!/usr/bin/python
# -*- coding: utf-8 -*-
PYTHONIOENCODING="UTF-8"

'''*
	Copyright (C) 2016 DudeHere

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

*'''

import time
import urllib
import math
import xbmc
from dudehere.routines import *
from dudehere.routines import plugin
from datetime import datetime, date
test = xbmc.__version__.split('.')
is_depricated = True if int(test[1]) < 19  else False
def str2bool(v):
	return v.lower() in ("yes", "true", "t", "1")

"""
	Need to move the walter service components into DHCR generic service to avoid need for double service
	Will do this at some point.
	
"""

SETTINGS_CHANGED = True

class WalterService():
	
	def __init__(self, *args, **kwargs):
		self.initiated = datetime.now()
		self.delay = 30
		self.timers = {}
		self.__enable_walter = False

	def log(self, message):
		plugin.log(message, 1)

	def update_settings(self):
		global SETTINGS_CHANGED
		if SETTINGS_CHANGED:
			self.__enable_walter = plugin.get_bool_setting('enable_walter')
			SETTINGS_CHANGED = False

	def get_setting(self, setting):
		return plugin.get_setting(setting)

	def get_bool_setting(self, setting):
		return  str2bool(plugin.get_setting(setting))

	def set_setting(self, setting, value):
		return plugin.set_setting(setting, value)

	def set_last_run(self, event, runtime):
		self.set_setting('last-run_'+event,runtime)

	def timestamp(self, d):
		return time.mktime(d.timetuple())
	
	def convert_setting_to_hours(self, t):
		h = 0
		timers = [8, 12, 24]
		h=timers[int(t)]
		return h
	
	def get_next_run(self, f, delay=0):
		seconds =  f * 3600
		delay = delay * 60;
		today = date.today()
		zero = self.timestamp(today)
		now = self.timestamp(datetime.now())
		delta = (now - zero) / (seconds)
		offset = math.ceil(delta)
		next =  offset * seconds + (zero + delay)
		next = datetime.fromtimestamp(next)
		return next
		
	def get_last_run(self, event):
		runtime=self.get_setting('last-run_'+event)
		if runtime:
			try:
				lastrun = datetime.strptime(runtime, '%Y-%m-%d %H:%M:%S.%f')
			except TypeError:
				lastrun = datetime.fromtimestamp(time.mktime(time.strptime(runtime, '%Y-%m-%d %H:%M:%S.%f')))
			return lastrun
		else:
			return self.initiated
	
	def clear_timers(self):
		self.log("Clearing timers...")
		self.set_last_run('tvshows', '')
		self.set_last_run('movies', '')
	
	def load_settings(self):
		self.log("Loading service settings...")
		self.update_timer = self.convert_setting_to_hours(self.get_bool_setting('update_timer'))
		self.update_tvshows = self.get_bool_setting('update_tvshows')
		self.update_movies = self.get_bool_setting('update_movies')
	
	def _run_kodi_cmd(self, params):
		cmd = 'RunPlugin(plugin://%s/?%s)' % (ADDON_ID, params)
		xbmc.executebuiltin(cmd)


	def setup_timers(self):
		self.log("Initiating timers...")
		if not self.enabled:
			return
		if self.update_tvshows:
			nextrun = self.get_next_run(self.update_timer)
			self.log("Next tv show update scheduled to run at %s" % nextrun)
			self.timers['tvshows'] = {}
			self.timers['tvshows']['lastrun'] = self.get_last_run('tvshows')
			self.timers['tvshows']['interval'] = self.update_timer
			self.timers['tvshows']['nextrun'] = nextrun
			self.timers['tvshows']['command'] = 'RunScript('+ ADDON_ID + ',mode=update_tvshows)'
			self.set_setting('next_tvshows_run', str(nextrun))
		else:
			self.timers['tvshows'] = None
	
		if self.update_movies:
			nextrun = self.get_next_run(self.update_timer, delay=5)
			self.log("Next movie update scheduled to run at %s" % nextrun)
			self.timers['movies'] = {}
			self.timers['movies']['lastrun'] = self.get_last_run('movies')
			self.timers['movies']['interval'] = self.update_timer
			self.timers['movies']['nextrun'] = nextrun
			self.timers['movies']['command'] = 'RunScript('+ ADDON_ID + ',mode=update_movies)'
			self.set_setting('next_movies_run', str(nextrun))
		else:
			self.timers['movies'] = None
			
			
	def evaluate_timers(self):
		now = self.timestamp(datetime.now())
		if self.update_tvshows:
			if now > self.timestamp(self.timers['tvshows']['nextrun']):
				self.execute('tvshows')
		if self.update_movies:
			if now > self.timestamp(self.timers['movies']['nextrun']):
				self.execute('movies')

	def execute(self, event):
		self.log("Executing: %s" % event)
		self.log(str(datetime.now()))
		self.timers[event]['lastrun'] =  datetime.now()
		self.timers[event]['nextrun'] = self.get_next_run(self.timers[event]['interval'])
		self.set_last_run(event, str(self.timers[event]['lastrun']))
		self.set_setting('last_%s_run' % event, str(self.timers[event]['lastrun']))
		self.set_setting('next_%s_run' % event, str(self.timers[event]['nextrun']))
		xbmc.executebuiltin(self.timers[event]['command'])
		self.log("Next %s update scheduled to run at %s" % (event, self.timers[event]['nextrun']))
		self.log("Waiting for next event...")
	
	def start(self):
		self.enabled = self.__enable_walter
		if self.enabled:
			self.log("Starting...")
			self.load_settings()
			self.clear_timers()
			self.setup_timers()
			self.log("Waiting for next event...")
			if self.get_bool_setting('run_on_startup'):
				self.execute('tvshows')
				self.execute('movies')
		
		
	def run(self):
		class Monitor(xbmc.Monitor):
			def onSettingsChanged(self):
				global SETTINGS_CHANGED
				SETTINGS_CHANGED = True
		monitor = Monitor()
		if is_depricated:
			while not xbmc.abortRequested:
				if self.enabled:
					self.evaluate_timers()
				if self.enabled != self.is_enabled():
					self.start()
				xbmc.sleep(2)
		else:
			while not monitor.abortRequested():
				if monitor.waitForAbort(10):
					break
				self.update_settings()
				if self.__enable_walter:
					self.evaluate_timers()
				if self.enabled != self.__enable_walter:
					self.start()

		self.log("Stopping...")

if __name__ == '__main__':
	Walter = WalterService()
	Walter.start()
	Walter.run()

